/**********************************************************************
 * tcp_socket_server.c                                  September 1999
 * Horms                                             horms@vergenet.net
 *
 * Open accpet a tcp connection from a client (we are the server)
 *
 * Licenced under the terms of the GNU LGPL
 *
 **********************************************************************/

#include "tcp_socket.h"

/*Keep track of the total number of connections in the parent process*/
unsigned int noconnection;


/**********************************************************************
 * tcp_socker_server_connect
 * Listen on a tcp port for incoming client connections 
 * When one is recieved fork
 * In the Child: close the listening file desctiptor
 *               return the file descriptor that is 
 *               a socket connection to the client
 * In the Server: close the socket to the client and loop
 * pre: port: port to listen to, an ascii representation of a number
 *            or an entry from /etc/services
 *      interface_address: If NULL bind to all interfaces, else
 *                         bind to interface(es) with this address.
 *      maximum_connections: maximum number of active connections
 *      to handle. If 0 then an number of connections is unlimited.
 *      return_from: pointer to a struct_in addr where the 
 *                   connecting client's IP address will
 *                   be placed. Ignored if NULL
 *      return_to: pointer to a in addr where the IP address the 
 *                 server accepted the connection on will be placed.
 *                 Ignored if NULL
 * post: Client sockets are returned in child processes
 *       In the parent process the function doesn't exit, other 
 *       than on error.
 *       if return_from is non-null, it is seeded with cleints address
 * return: open client socket, if connection is accepted.
 *         -1 on error
 **********************************************************************/

int tcp_socket_server_connect(
  const char *port, 
  const char *interface_address,
  const unsigned int maximum_connections,
  struct sockaddr_in *return_from,
  struct sockaddr_in *return_to
){
  int s;
  int g;
  int addrlen;
  struct sockaddr_in from;
  struct hostent *hp;

  extern unsigned int noconnection;

  if((s=socket(AF_INET, SOCK_STREAM, 0)) < 0){
    return(-1);
  }

  /* 
   * Set SO_REUSEADDR on the server socket s. Variable g is used
   * as a scratch varable.
   */
  g = 1;
  if(setsockopt(s,SOL_SOCKET,SO_REUSEADDR,(void *)&g,sizeof g) <0){
    return(-1);
  }

  from.sin_family = AF_INET;
  if((from.sin_port=tcp_socket_port_portno(port))==0){
    return(-1);
  }
  if(interface_address==NULL){
    from.sin_addr.s_addr = htonl(INADDR_ANY);
  }
  else {
    if((hp=gethostbyname(interface_address))==NULL){
      return(-1);
    }
    bcopy(hp->h_addr, (struct sockaddr *)&from.sin_addr, hp->h_length);
  }
  addrlen = sizeof(struct sockaddr_in);

  if(bind(s, (struct sockaddr *)&from, addrlen)<0){
    return(-1);
  }

  signal(SIGCHLD, (void(*)(int))tcp_socket_server_reaper);
  listen(s, 5);

  for(;;){
    if((g=accept(s, (struct sockaddr *)&from, &addrlen))>=0){
      if(maximum_connections && noconnection>=maximum_connections){
        close(g);
        continue;
      }
      if(fork()==0){
        if(close(s)<0){
          return(-1);
        }
        if(return_to!=NULL){
          addrlen=sizeof(struct sockaddr_in);
          if(getsockname(g, return_to, &addrlen)<0){
            printf("flim\n");
            return(1);
          }
        }
	if(return_from!=NULL){
          *return_from=from;
        }
        return(g);
      }
      else{
        noconnection++;
        if(close(g)){
          return(-1);
        }
      }
    }
  }
}


/**********************************************************************
 * tcp_socker_server_reaper
 * A signal handler that waits for SIGCHLD and runs wait3 to free
 * the resources of any exited children. This stops zombie processes
 * from hanging around.
 * pre: SIGCHLD is recieved by the process
 * post: Resoerces of any exited children are freed
 *       Signal Handler for SIGCHLD reset
 **********************************************************************/

void tcp_socket_server_reaper(void){
  int status;

  extern unsigned int noconnection;

  signal(SIGCHLD, (void(*)(int))tcp_socket_server_reaper);
  while(wait3(&status, WNOHANG, 0)>0){
    noconnection--;
  }
}
