/**********************************************************************
 * greeting.c                                              October 1999
 * Horms                                             horms@vergenet.net
 *
 * Protocol independent greeting
 *
 * Released under the terms of the GNU GPL
 *
 **********************************************************************/

#include "greeting.h"
#include "options.h"

/**********************************************************************
 * greeting
 * Send a greeting to the user
 * Pre: fd: file descriptor to write to
 *      protocol: Protocol in use
 *      message: Message to display
 *      flag: Flags as per greeting.h
 * Return 0 on success
 *        -1 on error
 **********************************************************************/

int greeting(const int fd, const protocol_t *protocol, flag_t flag){
  char *message=NULL;

  extern options_t opt;

  if((message=greeting_str(message, protocol, flag))==NULL){
    if(opt.debug){
      syslog(LOG_WARNING, "greeting: greeting_str");
    }
    return(-1);
  }
  if(protocol->write(
    fd, 
    NULL_FLAG, 
    NULL, 
    protocol->type[PROTOCOL_OK], 
    message
  )<0){
    if(opt.debug){
      syslog(LOG_WARNING, "greeting: protocol->write");
    }
    return(-1);
  }
  free(message);
  return(0);
}


/**********************************************************************
 * greeting_str
 * Produce greeting string
 * Pre: message: unallocated ponter to put greeting string in
 *      protocol: Protocol in use
 *      flag: Flags as per greeting.h
 * Return message string on success
 *        NULL on error
 **********************************************************************/

char * greeting_str(char *message, const protocol_t *protocol, flag_t flag){
  extern options_t opt;
  extern struct utsname *system_uname;

  if(flag&GREETING_ADD_NODENAME){
    if((message=cat_str(
      3,
      protocol->greeting_string,
      " ",
      system_uname->nodename
    ))==NULL){
      if(opt.debug){
        syslog(LOG_WARNING, "greeting_str: cat_str");
      }
      return(NULL);
    }
  }
  else{
    if((message=strdup(protocol->greeting_string))==NULL){
      if(opt.debug){
        syslog(LOG_WARNING, "greeting_str: strdup");
      }
      return(NULL);
    }
  }
  return(message);
}
