/**********************************************************************
 * options.h                                                   May 1999
 * Horms                                             horms@vergenet.net
 *
 * Read in command line options
 * Code based on man getopt(3)
 *
 * Released under the terms of the GNU GPL
 **********************************************************************/

#ifndef PERDITION_OPT_STIX
#define PERDITION_OPT_STIX

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>   /*For u_int32_t */
#include <perdition_adt.h>
#include <jain.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#include "../config.h"
#else
#define HAVE_GETOPT_LONG
#endif

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include "log.h"
#include "protocol.h"
#include "daemon.h"
#include "str.h"
#include "server_port.h"

#define OPT_SERVER_DELIMITER                ','

#define DEFAULT_AUTHENTICATE_IN              0
#define DEFAULT_BIND_ADDRESS                 NULL
#define DEFAULT_CLIENT_SERVER_SPECIFICATION  0
#define DEFAULT_CONFIG_FILE                  "/etc/perdition/perdition.conf"
#define DEFAULT_CONNECTION_LIMIT             0
#define DEFAULT_DEBUG                        0
#define DEFAULT_DOMAIN_DELIMITER             "@"
#define DEFAULT_GROUP                        "nobody"
#define DEFAULT_INETD_MODE                   0
#define DEFAULT_MAP_LIB \
  PREFIX "/lib/libperditiondb_gdbm.so"
#define DEFAULT_MAP_LIB_OPT                  NULL
#define DEFAULT_OUTGOING_SERVER              NULL
#define DEFAULT_PROTOCOL                     PROTOCOL_POP3
#define DEFAULT_STRIP_DOMAIN                 0
#define DEFAULT_SERVER_OK_LINE               0
#define DEFAULT_TIMEOUT                      1800 /*in seconds*/
#define DEFAULT_USERNAME                     "nobody"

typedef struct {
  int             authenticate_in;
  char            *bind_address;
  int             client_server_specification;
  char            *config_file;
  int             connection_limit;
  int             debug;
  char            *domain_delimiter;
  int             domain_delimiter_length;
  char            *group;
  int             inetd_mode;
  int             timeout;
  char            *listen_port;
  char            *map_library;
  char            *map_library_opt;
  char            *outgoing_port;
  dynamic_array_t *outgoing_server;
  int             protocol;
  int             server_ok_line;
  int             strip_domain;
  char            *username;
  flag_t          mask;
} options_t;

/*options_t.mask entries*/
#define MASK_AUTHENTICATE_IN             (flag_t) 0x00000001
#define MASK_BIND_ADDRESS                (flag_t) 0x00000002
#define MASK_CONNECTION_LIMIT            (flag_t) 0x00000004
#define MASK_DEBUG                       (flag_t) 0x00000008
#define MASK_DOMAIN_DELIMITER            (flag_t) 0x00000010
#define MASK_CLIENT_SERVER_SPECIFICATION (flag_t) 0x00000020
#define MASK_CONFIG_FILE                 (flag_t) 0x00000040
#define MASK_GROUP                       (flag_t) 0x00000080
#define MASK_INETD_MODE                  (flag_t) 0x00000100
#define MASK_LISTEN_PORT                 (flag_t) 0x00000200
#define MASK_MAP_LIB                     (flag_t) 0x00000400
#define MASK_MAP_LIB_OPT                 (flag_t) 0x00000800
#define MASK_OUTGOING_PORT               (flag_t) 0x00001000
#define MASK_OUTGOING_SERVER             (flag_t) 0x00002000
#define MASK_PROTOCOL                    (flag_t) 0x00004000
#define MASK_SERVER_OK_LINE              (flag_t) 0x00008000
#define MASK_STRIP_DOMAIN                (flag_t) 0x00010000
#define MASK_TIMEOUT                     (flag_t) 0x00020000
#define MASK_USERNAME                    (flag_t) 0x00040000

/*Flag values for options()*/
#define OPT_ERR         (flag_t) 0x1  /*Print error to stderr, enable help*/
#define OPT_CLEAR_MASK  (flag_t) 0x2  /*Set mask to 0*/
#define OPT_SET_MASK    (flag_t) 0x4  /*Add to mask as options are set*/
#define OPT_USE_MASK    (flag_t) 0x8  /*Don't accept options in the mask*/
#define OPT_SET_DEFAULT (flag_t) 0x10 /*Reset options to defaults before
                                           reading options passed*/
#define OPT_FILE        (flag_t) 0x20 /*Reading an options file*/
#define OPT_NOT_SET     (flag_t) 0x40 /*Option is not set, don't free*/
#define OPT_LIT         (flag_t) 0x80 /*Option is a litteral, don't free
                                            or copy, over-rides OPT_SET*/

#define OPT_FIRST_CALL  (flag_t) \
 OPT_ERR|OPT_CLEAR_MASK|OPT_SET_MASK|OPT_SET_DEFAULT

int options(int argc, char **argv, flag_t flag);
int options_set_mask(flag_t *mask, flag_t flag, flag_t mask_entry);
int log_options(void);
void usage(int exit_status);
dynamic_array_t *split_str_server_port(char *string, const char delimiter);

#endif
